/*
 * Title:        sag_connectivity_c.h
 * Description:  C-ABI data_t type
 * $Copyright (c) 2015-2017, 2019, 2022 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 */

/** @file
 *
 * Instead of including this file directly, include sag_connectivity_cpp.hpp if you need any types defined in this file. 
 */

// Contains the C ABI for connectivity plugins.

#ifndef _CMESSAGEDATA_H_
#define _CMESSAGEDATA_H_

#include <stdbool.h>
#include <stdint.h>

#include <sag_internal/c_macros.hpp>

/*
 * C-ABI for SAG connectivity plugins. This should not be used directly.
 * Instead use the C++ API in sag_connectivity_cpp.hpp.
 */
#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

/** A descriminator for the content of the data_t union */
enum sag_data_tag
{
	/** Empty */
	SAG_DATA_EMPTY,
	/** Bool */
	SAG_DATA_BOOLEAN,
	/** 64-bit float */
	SAG_DATA_DOUBLE,
	/** 64-bit signed int */
	SAG_DATA_INTEGER,
	/** 64-bit IEEE-754 decimal */
	SAG_DATA_DECIMAL,
	/** utf8-encoded const char* */
	SAG_DATA_STRING,
	/** list of data */
	SAG_DATA_LIST,
	/** map of data:data */
	SAG_DATA_MAP,
	/** byte-array (8-bit signed int) */
	SAG_DATA_BUFFER,
	/** a void* + deleter and copy function pointers */
	SAG_DATA_CUSTOM,
	/**
	 * \private
	 * Internal string optimization only for use by the correlator.
	 */
	SAG_DATA_CONST_STRING
};
typedef enum sag_data_tag sag_data_tag;

/**
 * \private
 */
enum sag_error_t
{
	/** No error */
	SAG_ERROR_OK,
	/** An exception was thrown */
	SAG_ERROR_EXCEPTION,
	/** Other failure condition */
	SAG_ERROR_FAILED,
	/** Unknown state */
	SAG_ERROR_UNKNOWN
};
/** \private */
typedef enum sag_error_t sag_error_t;

/** Log levels for logging to the host log file. */
enum sag_log_level_t
{
	/** Critical log messages */
	SAG_LOG_CRIT,
	/** Fatal errors */
	SAG_LOG_FATAL,
	/** Non-fatal errors */
	SAG_LOG_ERROR,
	/** Warnings */
	SAG_LOG_WARN,
	/** Informational messages (default) */
	SAG_LOG_INFO,
	/** Debug messages */
	SAG_LOG_DEBUG,
	/** Trace */
	SAG_LOG_TRACE
};
typedef enum sag_log_level_t sag_log_level_t;

/** Decimals are implemented with an underlying 64bit int conforming to IEEE
	754 decimal 64. This struct only provides access to the 64 bit data, which can be accessed with a 64-bit decimal
	library in order to perform arithmetic or convert to and from strings. 
	
	See also com::softwareag::connectivity::decimal_t which is how this should be referred to in customer code. 
*/
struct sag_underlying_decimal_t 
{
	/** The IEEE-754 data */
	uint64_t d;
};

typedef struct sag_underlying_decimal_t sag_underlying_decimal_t;

#include <sag_internal/c_types.hpp>

#include <sag_internal/c_functions.hpp>

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif //_CMESSAGEDATA_H_
